import 'package:blessing/screens/scaffold.dart';
import 'package:blessing/widgets/fade_transition_page.dart';
import 'package:flutter/material.dart';

import '../routing.dart';
import 'scaffold_body.dart';
import 'sign_in.dart';

class BlessingScreen {
  final ValueKey<String> key;
  final Widget widget;

  BlessingScreen(this.key, this.widget);
}

abstract class BlessingNavigator extends StatefulWidget {
  final GlobalKey<NavigatorState> navigatorKey;
  final BlessingScaffoldBody body;
  final List<NavigationItem> navigations;
  final SignInScreen signInScreen;

  const BlessingNavigator(
      {required this.navigatorKey,
      required this.body,
      required this.navigations,
      required this.signInScreen,
      super.key});

  @override
  State<BlessingNavigator> createState() => _BlessingNavigatorState();

  BlessingScreen? getScreen(ParsedRoute route);
}

class _BlessingNavigatorState extends State<BlessingNavigator> {
  final _signInKey = const ValueKey('Sign in');
  final _scaffoldKey = const ValueKey('Application scaffold');

  @override
  Widget build(BuildContext context) {
    final routeState = RouteStateScope.of(context);
    var screen = widget.getScreen(routeState.route);

    return Navigator(
      key: widget.navigatorKey,
      onPopPage: (route, dynamic result) {
        // if (route.settings is Page) {}
        return route.didPop(result);
      },
      pages: [
        if (routeState.route.pathTemplate == '/signin')
          FadeTransitionPage<void>(key: _signInKey, child: widget.signInScreen)
        else ...[
          FadeTransitionPage(
              key: _scaffoldKey,
              child: BlessingScaffold(
                  body: widget.body, navigations: widget.navigations)),
          if (screen != null)
            MaterialPage(key: screen.key, child: screen.widget)
        ]
      ],
    );
  }
}
