import 'package:flutter/material.dart';
import 'package:adaptive_navigation/adaptive_navigation.dart';

import '../routing.dart';
import 'scaffold_body.dart';

typedef NavigationItemHandle = bool Function(String path);

abstract class NavigationItem {
  final String title;
  final IconData icon;
  final String path;
  final NavigationItemHandle process;

  NavigationItem(this.path, this.title, this.icon, this.process);
}

class BlessingScaffold extends StatelessWidget {
  final BlessingScaffoldBody body;
  final List<NavigationItem> navigations;

  const BlessingScaffold(
      {super.key, required this.body, required this.navigations});

  @override
  Widget build(BuildContext context) {
    final routeState = RouteStateScope.of(context);
    final selectedIndex = _getSelectedIndex(routeState.route.pathTemplate);
    final destinations = navigations
        .map(
          (e) => AdaptiveScaffoldDestination(title: e.title, icon: e.icon),
        )
        .toList();

    return Scaffold(
        body: AdaptiveNavigationScaffold(
      selectedIndex: selectedIndex,
      body: body,
      onDestinationSelected: (idx) {
        if (navigations.length <= idx || idx < 0) {
          idx = 0;
        }
        routeState.go(navigations[idx].path);
      },
      destinations: destinations,
    ));
  }

  int _getSelectedIndex(String pathTemplate) {
    var idx =
        navigations.indexWhere((element) => element.process(pathTemplate));

    return idx == -1 ? 0 : idx;
  }
}
